@@ -110,11 +110,31 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
               required in libguestfs.");
 
   (* We use Augeas for inspection and conversion, so initialize it early.
-   * Calling debug_augeas_errors will display any //error nodes in
-   * debugging output if verbose (but otherwise it does nothing).
    *)
-  g#aug_init "/" 1;
-  debug_augeas_errors g;
+  let () =
+    let aug_save_backup = 1
+    and aug_no_load = 32 in
+    g#aug_init "/" (aug_save_backup + aug_no_load);
+
+    (* Exclude some lense includes which are problematic on a case-by-case
+     * basis.  Note the double quotes are part of the incl, and the incl
+     * must exactly match the definition in the Augeas lens file.
+     * - "/etc/lvm/archive/*.vg" because of RHEL-113820
+     *)
+    let removed_incls = [ {|"/etc/lvm/archive/*.vg"|} ] in
+    List.iter (
+      fun incl ->
+        let augpath = sprintf "/augeas/load//incl[%s]" incl in
+        let n = g#aug_rm augpath in
+        debug "convert_linux: removed %d incl(s) matching %s" n incl;
+    ) removed_incls;
+
+    g#aug_load ();
+
+    (* Calling debug_augeas_errors will display any //error nodes in
+     * debugging output if verbose (but otherwise it does nothing).
+     *)
+    debug_augeas_errors g in
 
   (* Clean RPM database.  This must be done early to avoid RHBZ#1143866. *)
   Array.iter g#rm_f (g#glob_expand "/var/lib/rpm/__db.00?");
@@ -131,25 +151,6 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
   (*----------------------------------------------------------------------*)
   (* Conversion step. *)
 
-  let uninstall_packages pkgs =
-    if pkgs <> [] then (
-      let cmd =
-        try Guest_packages.uninstall_command pkgs inspect.i_package_management
-        with
-        | Guest_packages.Unknown_package_manager msg
-        | Guest_packages.Unimplemented_package_manager msg ->
-          error "%s" msg
-      in
-      (try ignore (g#sh cmd)
-       with G.Error msg ->
-         warning (f_"could not uninstall packages ‘%s’: %s (ignored)")
-           (String.concat " " pkgs) msg
-      );
-      (* Reload Augeas in case anything changed. *)
-      Linux.augeas_reload g
-    )
-  in
-
   let rec do_convert () =
     augeas_grub_configuration ();
 
@@ -277,7 +278,7 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
         fun { G.app2_name = name } -> name = package_name
       ) inspect.i_apps in
     if has_guest_additions then
-      uninstall_packages [package_name];
+      uninstall_packages_nonfatal [package_name];
 
     (* Guest Additions might have been installed from a tarball.  The
      * above code won't detect this case.  Look for the uninstall tool
@@ -422,7 +423,7 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
       )
     );
 
-    uninstall_packages !remove;
+    uninstall_packages_nonfatal !remove;
 
     (* VMware Tools may have been installed from a tarball, so the
      * above code won't remove it.  Look for the uninstall tool and run
@@ -470,7 +471,7 @@ let convert (g : G.guestfs) source inspect i_firmware _ keep_serial_console _ =
     let pkgs = List.map (fun { G.app2_name = name } -> name) pkgs in
 
     if pkgs <> [] then (
-      uninstall_packages pkgs;
+      uninstall_packages_nonfatal pkgs;
 
       (* Installing these guest utilities automatically unconfigures
        * ttys in /etc/inittab if the system uses it. We need to put
@@ -1378,6 +1379,30 @@ fi
       "/etc/blkid/blkid.tab"; "/etc/blkid.tab";
       "/etc/lvm/cache/.cache"; "/etc/lvm/devices/system.devices"
     ];
+
+  (* This is a wrapper around Guestfs_packages.uninstall_command
+   * which catches errors and turns them into warnings, since
+   * uninstalling packages is best effort in virt-v2v.  It also
+   * reloads the Augeas configuration since removing packages might
+   * change /etc files.
+   *)
+  and uninstall_packages_nonfatal pkgs =
+    if pkgs <> [] then (
+      let cmd =
+        try Guest_packages.uninstall_command pkgs inspect.i_package_management
+        with
+        | Guest_packages.Unknown_package_manager msg
+        | Guest_packages.Unimplemented_package_manager msg ->
+          error "%s" msg
+      in
+      (try ignore (g#sh cmd)
+       with G.Error msg ->
+         warning (f_"could not uninstall packages ‘%s’: %s (ignored)")
+           (String.concat " " pkgs) msg
+      );
+      (* Reload Augeas in case anything changed. *)
+      Linux.augeas_reload g
+    )
   in
 
   do_convert ()
