@@ -22,7 +22,8 @@ type password =                 (** Use [None] for no password *)
   | AskForPassword              (** [password=-] *)
   | PasswordFile of string      (** [password=+file] *)
 
-val create_ssh : ?bandwidth:Types.bandwidth ->
+val create_ssh : ?name:string ->
+                 ?bandwidth:Types.bandwidth ->
                  ?cor:string ->
                  ?retry:bool ->
                  server:string ->
